package com.qmth.distributed.print.business.activiti;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.qmth.distributed.print.business.entity.ExamTask;
import com.qmth.distributed.print.business.service.ExamTaskService;
import com.qmth.teachcloud.common.contant.SpringContextHolder;
import com.qmth.teachcloud.common.entity.BasicCourse;
import com.qmth.teachcloud.common.entity.SysOrg;
import com.qmth.teachcloud.common.entity.SysUser;
import com.qmth.teachcloud.common.enums.ExceptionResultEnum;
import com.qmth.teachcloud.common.service.BasicCourseService;
import com.qmth.teachcloud.common.service.SysOrgService;
import com.qmth.teachcloud.common.service.SysUserService;
import com.qmth.teachcloud.common.util.ServletUtil;
import org.activiti.engine.delegate.DelegateTask;
import org.activiti.engine.delegate.TaskListener;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.data.domain.Example;

import javax.annotation.Resource;
import java.util.Arrays;
import java.util.List;
import java.util.Objects;
import java.util.Set;
import java.util.stream.Collectors;

/**
 * @Description: 教研室主任审批
 * @Param:
 * @return:
 * @Author: wangliang
 * @Date: 2021/7/29
 */
public class DirectorApproveListener implements TaskListener {
    private final static Logger log = LoggerFactory.getLogger(DirectorApproveListener.class);

    /**
     * 查找同级和以下人员
     *
     * @param delegateTask
     */
    @Override
    public void notify(DelegateTask delegateTask) {
        SysOrgService sysOrgService = SpringContextHolder.getBean(SysOrgService.class);
        ExamTaskService examTaskService = SpringContextHolder.getBean(ExamTaskService.class);
        BasicCourseService basicCourseService = SpringContextHolder.getBean(BasicCourseService.class);
        SysUserService sysUserService = SpringContextHolder.getBean(SysUserService.class);

        SysUser sysUser = (SysUser) ServletUtil.getRequestUser();
        String flowId = delegateTask.getProcessInstanceId();
        QueryWrapper<ExamTask> examTaskQueryWrapper = new QueryWrapper<>();
        examTaskQueryWrapper.lambda().eq(ExamTask::getFlowId, Long.parseLong(flowId));
        ExamTask examTask = examTaskService.getOne(examTaskQueryWrapper);

        QueryWrapper<BasicCourse> basicCourseQueryWrapper = new QueryWrapper<>();
        basicCourseQueryWrapper.lambda().eq(BasicCourse::getSchoolId, sysUser.getSchoolId())
                .eq(BasicCourse::getCode, examTask.getCourseCode());
        BasicCourse basicCourse = basicCourseService.getOne(basicCourseQueryWrapper);

        if (Objects.isNull(basicCourse)) {
            throw ExceptionResultEnum.ERROR.exception("课程数据为空");
        }
        if (Objects.isNull(basicCourse.getOrgId())) {
            throw ExceptionResultEnum.ERROR.exception("课程机构为空");
        }
        List<SysOrg> sysOrgList = sysOrgService.findByConnectByRootOrgId(basicCourse.getOrgId());
        Set<Long> orgIds = sysOrgList.stream().map(s -> s.getId()).collect(Collectors.toSet());

        QueryWrapper<SysUser> sysUserQueryWrapper = new QueryWrapper<>();
        sysUserQueryWrapper.lambda().eq(SysUser::getSchoolId, sysUser.getSchoolId())
                .in(SysUser::getOrgId, orgIds);
        List<SysUser> sysUserList = sysUserService.list(sysUserQueryWrapper);
        List<String> ids = sysUserList.stream().map(s -> String.valueOf(s.getId())).collect(Collectors.toList());
        //添加或签的人员，一人通过即可进入下一环节
        delegateTask.addCandidateUsers(ids);
    }
}
